#!/usr/bin/bash
set -eo pipefail

# FlightCtl Observability Configuration Reloader
# This script updates all observability configuration from service-config.yaml
# and ensures all services are running with the latest configuration.

LOG_TAG="flightctl-observability-reload"

log() {
    local level="$1"
    shift
    logger -t "$LOG_TAG" "$level: $*"
    echo "$(date '+%Y-%m-%d %H:%M:%S') [$level] $*" >&2
}

check_prerequisites() {
    log "INFO" "Checking prerequisites..."
    
    # Check if service-config.yaml exists
    if [ ! -f "/etc/flightctl/service-config.yaml" ]; then
        log "ERROR" "Configuration file /etc/flightctl/service-config.yaml not found"
        exit 1
    fi
    
    # Check what observability components are installed
    local has_full_observability=false
    local has_otel_only=false
    
    # Check if full observability stack is installed
    if [ -f "/usr/local/bin/flightctl-observability-config-reloader.sh" ]; then
        has_full_observability=true
    fi
    
    # Check if standalone OpenTelemetry collector is installed
    if [ -f "/usr/local/bin/flightctl-otel-collector-config-reloader.sh" ]; then
        has_otel_only=true
    fi
    
    if [ "$has_full_observability" = false ] && [ "$has_otel_only" = false ]; then
        log "ERROR" "No observability components found. Please install flightctl-observability or flightctl-otel-collector package"
        exit 1
    fi
    
    # For full observability stack, check core FlightCtl services
    if [ "$has_full_observability" = true ]; then
        local core_services=("flightctl-api.service" "flightctl-worker.service" "flightctl-db.service" "flightctl-kv.service")
        local missing_services=()
        
        for service in "${core_services[@]}"; do
            if ! systemctl is-active "$service" >/dev/null 2>&1; then
                missing_services+=("$service")
            fi
        done
        
        if [ ${#missing_services[@]} -gt 0 ]; then
            log "ERROR" "Core FlightCtl services are not running: ${missing_services[*]}"
            log "ERROR" "Please run 'make deploy-quadlets' first to start core services"
            exit 1
        fi
    fi
    
    log "INFO" "Prerequisites check passed"
}

reload_observability_config() {
    log "INFO" "Reloading observability configuration..."
    
    # Run the observability config reloader if available
    if [ -f "/usr/local/bin/flightctl-observability-config-reloader.sh" ]; then
        if ! /usr/local/bin/flightctl-observability-config-reloader.sh; then
            log "ERROR" "Failed to reload observability configuration"
            exit 1
        fi
        log "INFO" "Observability configuration reloaded successfully"
    else
        log "INFO" "Full observability stack not installed, skipping"
    fi
}

reload_otel_collector_config() {
    log "INFO" "Reloading OpenTelemetry collector configuration..."
    
    # Check if otel-collector config reloader exists and run it
    if [ -f "/usr/local/bin/flightctl-otel-collector-config-reloader.sh" ]; then
        if ! /usr/local/bin/flightctl-otel-collector-config-reloader.sh; then
            log "ERROR" "Failed to reload OpenTelemetry collector configuration"
            exit 1
        fi
        log "INFO" "OpenTelemetry collector configuration reloaded successfully"
    else
        log "INFO" "OpenTelemetry collector not installed, skipping"
    fi
}

wait_for_service_ready() {
    local service_name="$1"
    local max_wait="${2:-60}"
    local wait_time=0
    
    log "INFO" "Waiting for $service_name to be ready..."
    
    while [ $wait_time -lt $max_wait ]; do
        if systemctl is-active "$service_name" >/dev/null 2>&1; then
            log "INFO" "$service_name is ready"
            return 0
        fi
        sleep 2
        wait_time=$((wait_time + 2))
    done
    
    log "WARNING" "$service_name did not become ready within ${max_wait}s"
    return 1
}

verify_observability_services() {
    log "INFO" "Verifying observability services status..."
    
    local all_services=(
        "flightctl-prometheus.service"
        "flightctl-grafana.service"
        "flightctl-otel-collector.service"
        "flightctl-userinfo-proxy.service"
    )
    
    local services_to_check=()
    local failed_services=()
    local skipped_services=()
    
    # Determine which services to check based on what's installed
    for service in "${all_services[@]}"; do
        # Check if service unit file exists
        if ! systemctl list-unit-files "$service" >/dev/null 2>&1; then
            skipped_services+=("$service")
            continue
        fi
        
        services_to_check+=("$service")
    done
    
    # If no services are found, that's an error
    if [ ${#services_to_check[@]} -eq 0 ]; then
        log "ERROR" "No observability services found to verify"
        exit 1
    fi
    
    # Verify each service
    for service in "${services_to_check[@]}"; do
        # Wait for service to be ready
        if ! wait_for_service_ready "$service" 30; then
            failed_services+=("$service")
        fi
    done
    
    # Report results
    if [ ${#skipped_services[@]} -gt 0 ]; then
        log "INFO" "Skipped services (not installed): ${skipped_services[*]}"
    fi
    
    if [ ${#failed_services[@]} -gt 0 ]; then
        log "ERROR" "Failed services: ${failed_services[*]}"
        log "ERROR" "Some observability services failed to start properly"
        
        # Show service status for failed services
        for service in "${failed_services[@]}"; do
            log "ERROR" "Status for $service:"
            systemctl status "$service" --no-pager -l || true
        done
        
        exit 1
    fi
    
    log "INFO" "All installed observability services are running correctly"
}

show_service_endpoints() {
    log "INFO" "Observability service endpoints:"
    
    # Extract configuration values
    local config_file="/etc/flightctl/service-config.yaml"
    
    if command -v yq >/dev/null 2>&1; then
        local base_domain=$(yq e '.global.baseDomain // "localhost"' "$config_file")
        local endpoints_shown=false
        
        # Show Grafana if installed
        if systemctl list-unit-files "flightctl-grafana.service" >/dev/null 2>&1; then
            local grafana_port=$(yq e '.observability.grafana.published_port // "3000"' "$config_file")
            echo "  Grafana:    http://${base_domain}:${grafana_port}"
            endpoints_shown=true
        fi
        
        # Show Prometheus if installed (internal only)
        if systemctl list-unit-files "flightctl-prometheus.service" >/dev/null 2>&1; then
            echo "  Prometheus: Internal only (container: flightctl-prometheus:9090)"
            endpoints_shown=true
        fi
        
        # Show UserInfo proxy if installed and configured
        if systemctl list-unit-files "flightctl-userinfo-proxy.service" >/dev/null 2>&1; then
            local userinfo_upstream=$(yq e '.observability.userinfo_proxy.upstream_url // ""' "$config_file")
            if [ -n "$userinfo_upstream" ]; then
                echo "  UserInfo:   Internal only (container: flightctl-userinfo-proxy:8080)"
                endpoints_shown=true
            fi
        fi
        
        # Show OpenTelemetry collector info if installed
        if systemctl list-unit-files "flightctl-otel-collector.service" >/dev/null 2>&1; then
            local otel_grpc_port=$(yq e '.observability.otel_collector.grpc_port // "4317"' "$config_file")
            local otel_http_port=$(yq e '.observability.otel_collector.http_port // "4318"' "$config_file")
            echo "  OTEL Collector: grpc://${base_domain}:${otel_grpc_port}, http://${base_domain}:${otel_http_port}"
            endpoints_shown=true
        fi
        
        if [ "$endpoints_shown" = false ]; then
            echo "  No active observability services found"
        fi
    else
        echo "  Install 'yq' to see service endpoints"
    fi
}

main() {
    log "INFO" "Starting FlightCtl observability configuration reload..."
    
    check_prerequisites
    reload_observability_config
    reload_otel_collector_config
    verify_observability_services
    show_service_endpoints
    
    log "INFO" "FlightCtl observability configuration reload completed successfully!"
}

# Show usage if help is requested
if [ "$1" = "--help" ] || [ "$1" = "-h" ]; then
    cat << EOF
FlightCtl Observability Configuration Reloader

USAGE:
    $0 [OPTIONS]

DESCRIPTION:
    Updates observability configuration from /etc/flightctl/service-config.yaml
    and ensures all observability services are running with the latest configuration.

    This script supports both full observability stack and standalone OpenTelemetry 
    collector installations.

    This script will:
    1. Check prerequisites based on installed components
    2. Reload observability configuration templates (if full stack is installed)
    3. Reload OpenTelemetry collector configuration (if installed)
    4. Restart affected services
    5. Verify all services are running correctly
    6. Display available service endpoints

OPTIONS:
    -h, --help    Show this help message

EXAMPLES:
    # Reload observability configuration
    $0

    # Check status after configuration changes
    systemctl status flightctl-grafana.service
    systemctl status flightctl-prometheus.service

EOF
    exit 0
fi

main "$@" 